from M_Challenge.stage_data import StageData

print('please input the following information:)')
password = (input('Database password: '))
host = int(input('Host: '))
upload_data = StageData(password, host)
data = upload_data.save_to_database()
print(f"Data extracted, processed and staged!")



import json
from copy import deepcopy
import pandas as pd
import requests


class GetData:
    """
        This class is used to scrape world bank data in a json format. Data extracted is then transformed
        to a dataframe.

        class attributes:
            url (str): this specifies the API data is being extracted from.
            bank_data & bank_data: Empty lists to hold extracted data during processing.
        """

    def __init__(self):
        self.bank_data1 = []
        self.bank_data = []
        self.url = "http://api.worldbank.org/v2/country/?format=json&page="

    def extract_data(self):
        for i in range(1, 7):
            url = self.url + str(i)
            r = requests.get(url).json()
            self.bank_data.append(r)

    def clean_data(self):
        for item in self.bank_data:
            self.bank_data1.append(item[1])

    def write_to_json(self):
        with open('./world_bank.json', 'w') as f:
            json.dump(self.bank_data1, f, indent=4)

    def cross_join(self, left, right):
        """function to do a cartesian product"""
        new_rows = [] if right else left
        for left_row in left:
            for right_row in right:
                temp_row = deepcopy(left_row)
                for key, value in right_row.items():
                    temp_row[key] = value
                new_rows.append(deepcopy(temp_row))
        return new_rows

    def flatten_list(self, data):
        """function ensures JSON arrays are flattened. It uses a list of dictionaries comprising keys from one
        iteration before assigned to each of the list's values. """
        for elem in data:
            if isinstance(elem, list):
                yield from self.flatten_list(elem)
            else:
                yield elem

    def json_to_dataframe(self):
        def flatten_json(data, prev_heading=''):

            """Function will normalize deeply nested JSON."""

            if isinstance(data, dict):
                rows = [{}]
                for key, value in data.items():
                    rows = self.cross_join(rows, flatten_json(value, prev_heading + '.' + key))
            elif isinstance(data, list):
                rows = []
                for i in range(len(data)):
                    [rows.append(elem) for elem in self.flatten_list(flatten_json(data[i], prev_heading))]
            else:
                rows = [{prev_heading[1:]: data}]
            return rows

        self.extract_data()
        self.clean_data()
        self.write_to_json()
        world_bank_df = pd.DataFrame(flatten_json(self.bank_data1))

        return world_bank_df


if __name__ == '__main__':
    gd = GetData()
    gd.json_to_dataframe()



import pandas as pd


class ProcessCSV:
    """Loads csv files in to a dataframe by taking a list of csv files names.
     The dataframes created are joined on a common column between the 2 dataframes.
     Reorders newly created dataframe columns.

        Args:
            csv_files (list): list of csv files that will be read to dataframe
            df_list (list): Empty list to hold or contain dataframes convert from csv
            temp_col (list): Empty list to hold or contain columns that is being reordered.
    """

    def __init__(self, col_name: [], position: []):
        self.csv_files = ['GDPData', 'APIData']
        self.col_name = col_name
        self.position = position
        self.df_list = []
        self.temp_col = []

    def read_csv_to_df(self):
        for i in range(len(self.csv_files)):
            self.df_list.append(pd.read_csv(self.csv_files[i] + ".csv"))
        data1 = self.df_list[0].iloc[:, :65]
        data2 = self.df_list[1].iloc[:, :5]
        # Join dataset and drop redundant columns
        data = pd.merge(data1, data2, on='Country Code', how='inner').drop(['TableName'], axis=1)
        return data

    def reorder_df_columns(self):
        data = self.read_csv_to_df()
        dataframe = data.drop(columns=self.col_name)
        for x in range(len(self.col_name)):
            self.temp_col.append(data[self.col_name[x]])
        for y in range(len(self.position)):
            dataframe.insert(loc=self.position[y], column=self.col_name[y], value=self.temp_col[y])
        return dataframe


if __name__ == '__main__':
    pc = ProcessCSV(['Region', 'IncomeGroup', 'SpecialNotes'], [4, 5, 6])
    pc.reorder_df_columns()



from setuptools import setup
from setuptools import find_packages
from pathlib import Path


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='M_challenge',
    version='0.0.1',
    description='A tool for extracting data and reading csv into datafames details',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/VBamgbaye/M_Challenge',
    author='Victor Bamgbaye',
    author_email='bamgbayev@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=[]
)



import psycopg2
from sqlalchemy import create_engine

from M_Challenge.data_etl.data_grab import GetData
from M_Challenge.data_etl.process_csv import ProcessCSV


class StageData:
    """
        This class stages fully processed data treated from the data_grab module on to a postgresql database.
        It also loads and process csv files before data is staged.

        class attributes:
            tablename1 & 2 (str): Names assigned to the tables created in the database
            wb_data : invokes the class method GetData to extract data
            gpp_data :invokes the class method ProcessCsv to process csv file and covert to dataframe
            conn & engine (str): postgresql connection details
        """

    def __init__(self, password, host):
        self.wb_data = GetData()
        self.gdp_data = ProcessCSV(['Region', 'IncomeGroup', 'SpecialNotes'], [4, 5, 6])
        self.tablename1 = "world_bank"
        self.tablename2 = "gdp"
        self.password = password
        self.host = host
        self.engine = create_engine(f"postgresql+psycopg2://postgres:{password}@{host}:5432/finance")
        self.conn = psycopg2.connect(f"dbname=finance user=postgres password={password} host={host} port=5432")

    def create_tables(self):
        cursor = self.conn.cursor()
        cursor.execute("DROP TABLE IF EXISTS world_bank")
        cursor.execute(
            "CREATE TABLE world_bank "
            "(id VARCHAR(3), iso2Code VARCHAR(2), country_name VARCHAR(100), region_id "
            "VARCHAR(3), region_iso2code VARCHAR(2), region_value VARCHAR(100), admin_region_id VARCHAR(3),"
            "admin_region_iso2code VARCHAR(2), admin_region_value VARCHAR(100), income_level_id VARCHAR(3),"
            "income_level_iso2code VARCHAR(2), income_level_value VARCHAR(30), lending_type_id VARCHAR(3),"
            "lending_type_iso2code VARCHAR(2), lending_type_value VARCHAR(20), capital_city VARCHAR(20),"
            "longitude VARCHAR(10), latitude VARCHAR(10))")
        cursor.execute("DROP TABLE IF EXISTS gdp")
        cursor.execute(
            "CREATE TABLE gdp "
            "(country_name VARCHAR(50), country_code VARCHAR(3), indicator_name VARCHAR(30),"
            "indicator_code VARCHAR(15), year_1999 int, year_2000 int, year_2001 int, year_2002 int, year_2003 int, "
            "year_2004 int, year_2005 int, year_2006 int, year_2007 int, year_2008 int, year_2009 int, year_2010 int, "
            "year_2011 int, year_2012 int, year_2013 int, year_2014 int, year_2015 int, year_2016 int, year_2017 int, "
            "year_2018 int, year_2019 int, year_2020 int)")
        self.conn.commit()

    def save_to_database(self):
        self.create_tables()
        self.wb_data.json_to_dataframe().to_csv('./world_bank_data.csv')
        self.wb_data.json_to_dataframe().to_sql(self.tablename1, self.engine, if_exists='replace', index=False)
        self.gdp_data.reorder_df_columns().to_sql(self.tablename2, self.engine, if_exists='replace', index=False)
        print(self.gdp_data.reorder_df_columns())
        print(f"Tables: {self.tablename1} & {self.tablename2} created. Data staged! ")


if __name__ == '__main__':
    sd = StageData(input('Enter password: '), input('Enter host: '))
    sd.save_to_database()

# sd = StageData(input('Enter password: '), input('Enter host: '))
# sd.save_to_database()
